% example1.m
% Regression Example Using Simulated Data
%John C Frain
%19 November 2006
%values for simulation
nsimul=50;
beta=[5,1,.1]';
%
% Step 1 Prepare and process data for X and y matrices/vectors
%
x1=ones(nsimul,1); %constant
x2=[1:nsimul]'; %trend
x3=rand(nsimul,1)*2 +3; % Uniform(3,5)
X=[x1,x2,x3];
e=randn(nsimul,1)*.2; % N(0,.04)
y= X * beta +e ; %5*x1 + x2 + .1*x3 + e;
%
[nobs,nvar]=size(X);
%
% Estimate Model
ols.betahat=(X'*X)\X'*y % Coefficients
ols.yhat = X * ols.betahat; % beta(1)*x1+beta(2)*x2+beta(3)*x;
ols.resid = y - ols.yhat; % residuals
ols.ssr = ols.resid'*ols.resid; % Sum of Squared Residuals
ols.sigmasq = ols.ssr/(nobs-nvar); % Estimate of variance
ols.covbeta=ols.sigmasq*inv(X'*X); % Covariance of beta
ols.sdbeta=sqrt(diag(ols.covbeta));% st. dev of beta
ols.tbeta = ols.betahat ./ ols.sdbeta; % t-statistics of beta
ym = y - mean(y);
ols.rsqr1 = ols.ssr;
ols.rsqr2 = ym'*ym;
ols.rsqr = 1.0 - ols.rsqr1/ols.rsqr2; % r-squared
ols.rsqr1 = ols.rsqr1/(nobs-nvar);
ols.rsqr2 = ols.rsqr2/(nobs-1.0);
if ols.rsqr2 ~= 0;
ols.rbar = 1 - (ols.rsqr1/ols.rsqr2); % rbar-squared
else
ols.rbar = ols.rsqr;
end;
ols.ediff = ols.resid(2:nobs) - ols.resid(1:nobs-1);
ols.dw = (ols.ediff'*ols.ediff)/ols.ssr; % durbin-watson
fprintf('R-squared      = %9.4f \n',ols.rsqr);
fprintf('Rbar-squared   = %9.4f \n',ols.rbar);
fprintf('sigma^2        = %9.4f \n',ols.sigmasq);
fprintf('S.E of estimate= %9.4f \n',sqrt(ols.sigmasq));
fprintf('Durbin-Watson  = %9.4f \n',ols.dw);
fprintf('Nobs, Nvars    = %6d,%6d \n',nobs,nvar);
fprintf('****************************************************\n \n');
% now print coefficient estimates, SE, t-statistics and probabilities
%tout = tdis_prb(tbeta,nobs-nvar); % find t-stat probabilities - no
%tdis_prb in basic MATLAB - requires leSage toolbox
%tmp = [beta sdbeta tbeta tout]; % matrix to be printed
tmp = [ols.betahat ols.sdbeta ols.tbeta]; % matrix to be printed
% column labels for printing results
namestr = ' Variable';
bstring = ' Coef.';
sdstring= 'Std. Err.';
tstring = ' t-stat.';
cnames = strvcat(namestr,bstring,sdstring, tstring);
vname = ['Constant','Trend' 'Variable2'];
fprintf('%12s %12s %12s %12s \n',namestr, ...
bstring,sdstring,tstring)
fprintf('%12s %12.6f %12.6f %12.6f \n',...
' Const',...
ols.betahat(1),ols.sdbeta(1),ols.tbeta(1))
fprintf('%12s %12.6f %12.6f %12.6f \n',...
' Trend',...
ols.betahat(2),ols.sdbeta(2),ols.tbeta(2))
fprintf('%12s %12.6f %12.6f %12.6f \n',...
' Var2',...
ols.betahat(3),ols.sdbeta(3),ols.tbeta(3))