% example2par.m
% Simple parallel extension by Michael Curran of:
% MATLAB Simulation Example
%John C Frain
%19 November 2006
%
% For the parallel extension to work, first of all you need to fire up the
% processors via the command 
% matlabpool 
% Once the workers are on, parfor loops will work. To turn off this
% function, use the command
% matlabpool close
% Note that you can specifiy the number of workers (subject to the number
% of cores in your CPU) via the command
% matlabpoo open 2
% In this case "2" is the number of workers I have chosen to fire up.
%
% The data files x20.csv, x100.csv, x500.csv and x2500.csv were generated
% using the code below

% matlabpool % you can uncomment this when you want to make the code switch
% on the extra processors automatically
%tic % uncomment this when you want to start timing the process from here

%Generate Data
x20 = 100*rand(20,1)
save('x20.csv','x20','-ASCII','-double')
x100 = 100*rand(100,1)
save('x100.csv','x100','-ASCII','-double')
x500 = 100*rand(500,1)
save('x500.csv','x500','-ASCII','-double')
x2500 = 100*rand(200,1)
save('x2500.csv','x2500','-ASCII','-double')
clear
nsimul=10000;
BETA20=zeros(nsimul,1); % vector - results of simulations with 20 obs.
x=load('-ascii', 'x20.csv'); % load xdata
X=[ones(size(x,1),1),x]; % X matrix note upper case X
beta = [ 10;2]; % true values of coefficients
%
%parfor ii = 1 : nsimul;
for ii = 1 : nsimul;
eps = 20.0*randn(size(X,1),1); % simulated error term
y = X * beta + eps; % y values
betahat = (X'*X)\X'*y; % estimate of beta
BETA20(ii,1)=betahat(2);
end
fprintf('Mean and st. dev of 20 obs simulation %6.3f %6.3f\n' ...
,mean(BETA20),std(BETA20))
%hist(BETA20,100)

BETA100=zeros(nsimul,1);
x=load('-ascii', 'x100.csv'); % load xdata
X=[ones(size(x,1),1),x]; % X matrix note upper case X
beta = [ 10;2]; % true values of coefficients
%
%parfor ii = 1 : nsimul;
for ii = 1 : nsimul;
    eps = 20.0*randn(size(X,1),1); % simulated error term
    y = X * beta + eps; % y values
    betahat = inv(X'*X)*X'*y; % estimate of beta
    BETA100(ii,1)=betahat(2);
end
fprintf('Mean and st. dev of 100 obs simulation %6.3f %6.3f\n', ...
mean(BETA100),std(BETA100))

BETA500=zeros(nsimul,1);
x=load('-ascii', 'x500.csv'); % load xdata
X=[ones(size(x,1),1),x]; % X matrix note upper case X
beta = [ 10;2]; % true values of coefficients
%
%parfor ii = 1 : nsimul;
for ii = 1 : nsimul;
eps = 20.0*randn(size(X,1),1); % simulated error term
y = X * beta + eps; % y values
betahat = inv(X'*X)*X'*y; % estimate of beta
BETA500(ii,1)=betahat(2);
end
fprintf('Mean and st. dev of 500 obs simulation %6.3f %6.3f\n', ...
mean(BETA500),std(BETA500))

BETA2500=zeros(nsimul,1);
x=load('-ascii', 'x2500.csv'); % load xdata note use of lower case x as vector
X=[ones(size(x,1),1),x]; % X matrix note upper case X
beta = [ 10;2]; % true values of coefficients
%
%parfor ii = 1 : nsimul;
for ii = 1 : nsimul;
eps = 20.0*randn(size(X,1),1); % simulated error term
y = X * beta + eps; % y values
betahat = inv(X'*X)*X'*y; % estimate of beta
BETA2500(ii,1)=betahat(2);
end
fprintf('Mean and st. dev of 2500 obs simulation %6.3f %6.3f\n', ...
mean(BETA2500),std(BETA2500))

n=hist([BETA20,BETA100,BETA500,BETA2500],1.4:0.01:2.6);
plot((1.4:0.01:2.6)',n/nsimul);
h = legend('Sample 20','Sample 100','Sample 500','Sample 2500');
%toc % uncomment this when you want to display the process time to here
% matlabpool close % you can uncomment this if you switched on matlabpool