% example6.m
% Tobit Estimation by Maximum Likelihood
% John C Frain
% 27 October 2009
%

%% Load and process data
clear;
load cigarette
% set up variables
y = sharetob;
[nobs, dump] = size(tobacco) ;
X = [ones(nobs,1), age, nadults, nkids, nkids2, lnexp, age.*lnexp, nadults.*lnexp];
[nobs, k]=size(X);

%% Estimate starting values using OLS
beta0 = ( X'*X)\X'*y ;
sd = sqrt((y-X*beta0)'*(y-X*beta0))/(nobs-1);
parm = [beta0
sd ];

%% Set up dfp min defaults - change maximum number of iterations
info.maxit = 1000;

%% Call optimisation routine
result2 = dfp_min('to_liked',parm,info,y,X);                                           

%% Extract results
beta = result2.b;
sdbeta = sqrt(diag(inv(result2.hess)));
zstat = beta ./ sdbeta;
%row names
rnames = 'b1';
for i=2:k;
bstring = ['b' num2str(i)];
rnames = strvcat(rnames,bstring);
end;
rnames = strvcat(rnames,'sigma');
disp('Estimates of tobit')
disp(' coef sd t-stat')
for ii=1:k+1
fprintf('%s%10.5f%10.5f%10.5f\n',rnames(ii,:),beta(ii),sdbeta(ii),zstat(ii))
end