%% loopplotfunc.m
% This m-file describes basic loops, plots and functions. With regards to
% functions, it calls the function myfunction, which takes two matrices as
% arguments (inputs) and produces the trace, determinant and transpose of
% the sum of the two matrices as output.

%% Calling a function
X=[1 2;3 4];
Y=[5 6;7 8];
[trc deter TRAN]=myfunction(X,Y);
disp('X:');
disp(X);
disp('Y:');
disp(Y);
disp('Trace of X+Y:');
disp(trc);
disp('Determinant of X+Y:');
disp(deter);
disp('Transpose of X+Y:');
disp(TRAN);

%% Basic plots and the for and if loops
n=randn(50,1);
u=rand(50,1);
x=1:50;
plot(x,n,'b-o',x,u,'r--x'); legend('Normal(0,1)','Uniform[0,1)'); ...
title('Normal & Uniform random numbers'); ylabel('Normal/Uniform'); ...
xlabel('Observation');
x=0:1/100:10;
hold on;
for i=1:9
    subplot(3,3,i);
    plot(x,i);
    if i==1
        title('i=1');
    elseif i==2
        title('i=2');
    elseif i==3
        title('i=3');
    elseif i==4
        title('i=4');
    elseif i==5
        title('i=5');
    elseif i==6
        title('i=6');
    elseif i==7
        title('i=7');
    elseif i==8
        title('i=8');
    else
        title('i=9');
    end
end
hold off;

%% The while loop and saving plots during a loop
grid on
NAME={'i=1','i=2','i=3','i=4','i=5'};
N=5;
i=1;
while i<=N
    fprintf('Line number %d.\n',i);
    figFilename=strcat('plot',num2str(i));
    plot(x,i); title(NAME(i));
    saveas(gcf,figFilename,'fig')
    i=i+1;
end

%% The switch loop
method=input('What method should be used? linear/bilinear/quadratic/cubic/nearest: ','s');
switch lower(method)
    case {'linear' 'bilinear'}
        disp('Method is linear.')
    case 'quadratic'
        disp('Method is quadratic.')
    case 'cubic'
        disp('Method is cubic.')
    case 'nearest'
        disp('Method is nearest.')
    otherwise
        disp('Unknown method.')
end

%% Elementary Plots (section 5 in Frain, 2010)
% values for simulation
nsimul=50;
beta=[5,1,.1]';
%
x1=ones(nsimul,1); %constant
x2=[1:nsimul]'; %trend
x3=rand(nsimul,1)*2+3; % Uniform(3,5)
x=[x1,x2,x3];
e=randn(nsimul,1)*.2; % N(0,.04)
y = x*beta+e; %5*x1 + x2 + .1*x3 + e;
%
[nobs,nvar]=size(x);
betahat = inv(x'*x)*x'*y %g
yhat = x * betahat; % beta(1)*x1-beta(2)*x2-beta(3)*x;
resid = y - yhat;
plot(x2,resid)
title('Graph Title')
xlabel('Time')
ylabel('Residual')
% this repeats the earlier OLS simulation, opens a graph window, dras a
% graph of the residuals against the trend in the ols-simulation exercise,
% puts a title ont he graph and labels the x and y axes. The vectors x2 and
% resid must have the same dimension. You can save the graph in eps format
% and import it into documents relatively easily.

%% More user written functions in MATLAB (Section 7 in Frain, 2010)
% One of the most useful facilities in MATLAB is the facility to write ones
% own functions and use them in the same way as a native MATLAB function.
% We are already familiar with m-files which contain lists of MATLAB
% instructions. Such files are known as script files and allow us to do
% repeat analysis without having to retype all the instructions. Suppose we
% wish to write a function that estimates the denstiy function of a normal
% distribution:
%
% $$\frac{1}{\sqrt{2\pi}\sigma}\exp{-\frac{(x-\mu)^2}{2\sigma^2}$$

 
% Look at the file normaldensity.m, which does exactly this. Note:
% (1) The file starts with the keyword function. This is to indicate that
% this m-file is a function definition.
% (2) In the first line the f indicates that the value of f when the file
% has been "run" is the value that will be returned.
% (3) The function is called with normdensity(z,mu,sigma) where z mu and
% sigma are given values in calling the function.
% (4) The commented lines immediately after the first line are a help
% system for the function.
% (5) All variables within a function are local to the function. Thus if
% there is a variable within a function called x and one in the program
% with the same name the one inthe program is used when the program is in
% operation. Once the program has been run the value the program is
% forgotten and the value outside the program is used.
% The use of the function normdensity can be demonstrated as follows - 
% Get help for normdensity function:
help normdensity
% Evaluate Standard NOrmal density function at zero
normdensity(0,0,1)
% Plot standard normal density function
fplot('normdensity(x,0,1)',[-3,3])