%% matrices.m
% This first sample session presents a rudimentary overview of many of the
% basic matrix manipulation commands that you will encounter while learning
% MATLAB.
%%


% This is a comment. Anything written to the right of a percentage symbol %
% is not read directly by MATLAB - this is helpful for commenting on your
% code so that you can recall what it does in the future or explain it to
% someone else who is using your code.

%% Preliminaries
clear all % clears all variables
close all % closes all extra open windows
clc % clears command window
clf % clears figure window

%% Matrix entry
a = 1
a = [1 2 3 4] % produces a 1x4 row vector
a = [1 2 3 4]; % ; suppresses output. Leave it out if you do not wish to suppress it.
% To get a 4x1 column vector, we could do either:
a = [1; 2; 3; 4]
% OR take the transpose:
a = [1 2 3 4]'
% To get a 3x2 matrix:
A = [1,2;3,4;5,6];
% To the value of a variable "a" that has previously been assigned a value, type:
a
m = 5
n = 3
A = ones(m,n) % creates an m x n array of ones
A = zeros(m,n) % creates an m x n array of zeros
A = nan(m,n) 
% creates an m x n array of nan's (Not A Number) - useful for initialising
% matrices
size(A) % returns the size of the array

% Referring to parts of a matrix and joining matrices
a=[4 5 6 7 8];
a(3)
A=[1,2,3;-4,5,6;7,8,9]
A(3,2)
A(:,2)
A(3,:)
A(:,:)
A(3,2:3)
A(end,2:end)
% Submatrices
B=[1 2 3 4;5 6 7 8;9 10 11 12]
B(1:2,[1,4])=[90,91;92 93]
B=[90,91;92 93]
B(1:2,1:2)=1
B=[1 2 3 4;5 6 7 8;9 10 11 12]
B(1:2,1:2)=1
% Stacking
B=[1 2;3 4]
A=[1 2;3 4]
B=[5 6;7 8]
Z=[A,B,(15:16)'] % horizontal
horzcat(A,B,(15:16)')
cat(2,A,B,(15:16)')
Z=[A',B',(15:16)']' % vertical
vertcat(A,B,(15:16))
cat(1,A,B,(15:16))
cat(3,A,B)
cat(4,A,B)
a={'abcde','fgh'};
b={'ijk','lmno'};
ab = strcat(a,b)
c={'R'};
abc=strcat(a,b,c)
a='Hello   ';
b='World';
using_strcat = strcat(a,b)
using_arrayop = [a,b] % note equivalence to horzcat(a,b)
s1='first';
s2='second';
s3='string';
s4='matrix';
S1 = strvcat(s1,s3,s4)
S2 = strvcat(s2,s3,s4)
S3 = strvcat(S1,S2)

%% Matrix operations
X = [1 2;3 4]
Y = [5 6;7 8]
X+Y
X-Y
% With matrix multiplication, their dimensions must conform
X*Y
Z = [1 2;3 4;5 6]
X*Z % This should produce an error message since the matrix dimensions do not agree here
inv(X)
X*inv(X) % checks inverse
Y*inv(X)
Y/X % equivalent to Y*inv(X)
inv(X)*Y
X\Y % equivalent but better for regression
% To multiply elementwise (so each element of matrix X only multiplies the
% corresponding element of matrix Y):
X.*Y
X./Y % element by element division
Z = [1 2;0 4];
X./Z % watch out dividing by zero: get infinity since it is undefined
m=2;
X+m
X-m
X*m
X/m
X^m
X*X
% element by element exponent
Z=[1 2;2 1]
X
X.^Z
X = [1 2;3 4]
I = eye(2) % 2x2 identity matrix
I = eye(2,2) 
% more general: can extend to any dimension and puts identity matrix
% starting with 1st row, 1st column
I = eye(2,3)
I = eye(3,2)
I = eye(2)
kron(X,I) % Kronecker product X Kronecker I
kron(I,X) % Kronecker product I Kronecker X
exp(X)
log(X)
exp(log(X))
sqrt(X)
det(X) % determinant
trace(X) % trace
% MATLAB returns complex numbers when you use negative numbers in arguments
% of logs and square roots, while many other packages produce errors. This
% may not be what you want, so be aware.
Z = [1 -4]
log(Z)
% diag(X) where X is a square matrix will yield a column vector of the
% diagonal entries of X. diag(a) where a is a column vector will yield a
% matrix where the diagonal entries are the elements of a and the
% off-diagonal elements are all zero.
z=diag(X)
A=diag(a)
A=[1 2 3 4
    5 6 7 8
    9 10 11 12] % yet another way of entering matrices!
rank(A)
sum(z) % If z is a vector, sum(z) returns the sum of its elements
sum(A) 
% If A is a matrix, sum(A) returns the sum of the elements of each column,
% columnwise
prod(z) % If z is a vector, prod(z) returns the product of its elements
prod(A) 
% If A is a matrix, prod(A) returns the product of the elements of each
% column, columnwise
% reshaping an array with exactly m*n elements into an m x n array
X = [1 2 3; 4 5 6]
reshape(X,3,2)
% blockdiagonal
A = [1 2;3 4]
B=5
C=[6 7 8;9 10 11;12 13 14]
blkdiag(A,B,C)
blkdiag(A,B,C,X)
% e-values and e-vectors
eig(A)
[V,D]=eig(A)
Test=A*V
Test./V
% Sequences
[0:.2:1]
[1:5]
[1:5]'
% Random number generators:
x=rand(5) % generates 5x5 matrix of uniform random numbers on [0,1)
x=rand(5,1) % generates 5x1 column vector of uniform random numbers on [0,1)
x=randn(1,5) 
% generates 1x5 row vector of random numbers drawn from a standard normal
% distribution, N(0,1)

%% Formatting displays
pi
format loose % allows blank lines (default)
pi
format compact % suppresses blank lines
pi
format loose
format long 
% 15 decimal places for double, 7 decimal places single (you can specify a
% number to be single by wrapping single around it, e.g. single(pi))
pi
format short % default, 4 decimal places
pi
format short e % scientific format
pi
format long e
pi
format short g
pi
format long g
pi
format short eng 
% engineering format: 4 decimal places and a power that is a multiple of 3
pi
format long eng
% engineering format: 14 decimal places and a power that is a multiple of 3
pi
format bank % currency format (2 decimal places)
pi
format hex % hexadecimal representation
pi
format rat % ratio of small integers
pi
format short g

%% Some input and output functions
disp('Hello world!');
fprintf('Pi is about %6.2f\n',pi) 
% display at least 6 digits of which 2 will be decimals in floating point
% (f) and move to next line (\n).
fprintf('A more accurate approximation for pi is %17.16f\n',pi)
approx=input('What is your guess for pi? ','s')
% Next bit is from section 3.6 on formatted output
% The MATLAB function fprintf() may be used to produce formatted output on
% screen. It is only one of a large number of C-sytle input/output
% functions in C. These allow considerable flexibility in sending formatted
% materila to the screen or to a file. The MATLAB help files give details
% of the facilities available. If further information is required, one
% might consult Kernighan & Ritchie: "The C Programming Language" - I would
% recommend this as the best introductory book to C programming. C was
% created by Dennis Ritchie.
% Sample MATLAB program demonstrating Formatted Output:
clear
degrees_c = 10:10:100;
degrees_f = (degrees_c * 9/5)+32;
fprintf('\n\n Conversion from degrees Celsius \n');
fprintf('     to degrees Fahrenheit\n\n');
fprintf('     Celsius   Fahrenheit\n');
for ii = 1:10;
    fprintf('%12.2f%12.2f\n',degrees_c(ii),degrees_f(ii));
end
%
fprintf(...
    '\n\n%5.2f degrees Celsius is equivalent of %5.3f degrees fahrenheit\n',...
    degrees_c(1),degrees_f(1))
% Note the following:
% (1) The first argument of the fprintf() function is a kid of format
% statement included within ' marks.
% (2) The remaining arguments are a list of ariables or items to be printed
% separated by commas.
% (3) Within the format string there is text which is produced exactly as
% set down. There are also statements like %m.nf which produces a decimal
% number which is allowed m columns oif output and has n places of
% decimals. These are applied in turn to the items in the list to be
% printed.
% (4) This f format is used to output floating point numbers there are a
% considerable number of other specifiers to ouptut characters, strings and
% number in formats other than floating point.
% (5) If the list to be printed is too long the formats are recycled.
% (6) Note the use of \n which means skip to the next line. THIs is
% essential.

% Next bit creates a data file of 1x5 vector of standard Normal distributed
% random
% variables and then reads them to the vector a:
% (i) random standard normal generator
data = randn(1,5) % this is the data
% (ii) write these to the file 'data.dat'
fileID = fopen('data.dat', 'w+'); % creates id for the data file
fprintf(fileID, '%f', data); % writes data to the file data.dat
frewind(fileID); % return to beginning of file
% (iii) read data to array A
a = fscanf(fileID, ['%f' '%f' '%f' '%f' '%f'])
a=a' % transpose so a matches orientation of data
fclose(fileID); % close file

%% Special values
pi
exp(1)
disp('Year Month Day Hours Minutes Seconds');
clock
c=clock; % Year Month, Day, Hours, Minutes, Seconds
s='Year        Month    Day      Hours    Minutes   Seconds';
fprintf('%f %f %f %f %f %f\n%s\n',c(1),c(2),c(3),c(4),c(5),c(6),s);
tic
toc

%% Exercise: go through examples in sections 2.2, 2.3, 2.4 and 2.5 of Frain (2010).
% I have included the files example1.m, example2.m and example3.m
% Note in particular the use of ols. in front of variable names. Also note
% the use of ... to continue a command onto a new line. Finally note the
% use of save, load, mean, std, hist, plot, legend, title, xlabel, ylabel
% and axis.